-- Ace3 init
GottaMogEmAll = LibStub("AceAddon-3.0"):NewAddon("GottaMogEmAll", "AceConsole-3.0", "AceEvent-3.0")
AceGUI = LibStub("AceGUI-3.0")

local GameTooltip = GameTooltip

-- Event subscription
-- ZONE_CHANGED_NEW_AREA
function GottaMogEmAll:AutoOpen()
	if(GottaMogEmAll:GetAutoOpen() == true) then
		C_Timer.After(1, function() GottaMogEmAll:FromInstance() end)
	end
end
GottaMogEmAll:RegisterEvent("ZONE_CHANGED_NEW_AREA", "AutoOpen")

-- Main Frame
mainFrame = AceGUI:Create("Frame")
mainFrame:SetTitle("Gotta Mog 'Em All")
mainFrame:SetLayout("Flow")
mainFrame:SetCallback("OnClose", function() GottaMogEmAll:Hide() end)

-- DressUpModel frame
local model = CreateFrame("DressUpModel")

local fromInstance = false -- How CheckItems() was called

-- Inventory slot names
local InventorySlots = {
    ['INVTYPE_HEAD'] = 1,
    ['INVTYPE_NECK'] = 2,
    ['INVTYPE_SHOULDER'] = 3,
    ['INVTYPE_BODY'] = 4,
    ['INVTYPE_CHEST'] = 5,
    ['INVTYPE_ROBE'] = 5,
    ['INVTYPE_WAIST'] = 6,
    ['INVTYPE_LEGS'] = 7,
    ['INVTYPE_FEET'] = 8,
    ['INVTYPE_WRIST'] = 9,
    ['INVTYPE_HAND'] = 10,
    ['INVTYPE_CLOAK'] = 15,
    ['INVTYPE_WEAPON'] = 16,
    ['INVTYPE_SHIELD'] = 17,
    ['INVTYPE_2HWEAPON'] = 16,
    ['INVTYPE_WEAPONMAINHAND'] = 16,
    ['INVTYPE_RANGED'] = 16,
    ['INVTYPE_RANGEDRIGHT'] = 16,
    ['INVTYPE_WEAPONOFFHAND'] = 17,
    ['INVTYPE_HOLDABLE'] = 17,
}

-- Player info
local localizedClass, englishClass, classIndex = UnitClass("player")

-- Chat command options
local options = {
    name = "Gotta Mog 'Em All",
    handler = GottaMogEmAll,
    type = 'group',
    args = {
        autoopen = {
            type = 'toggle',
            name = 'Auto open',
            desc = 'Outputs missing items on instance enter.',
            set = 'ToggleAutoOpen',
            get = 'GetAutoOpen',
        },
		toggle = {
			type = "toggle",
			name = "Toggle addon",
			desc = "Shows/hides the addon.",
			set = 'ToggleDisplay',
			get = 'IsShown'
		},
		dng = {
			type = "execute",
			name = "Request Dungeon",
			desc = "Sends a request for the current dungeon.",
			func = "FromInstance"
		},
		show = {
			type = "execute",
			name = "Show",
			desc = "Shows the addon.",
			func = "Show"
		},
		hide = {
			type = "execute",
			name = "Hide",
			desc = "Hides the addon.",
			func = "Hide"
		},
    },
}
LibStub("AceConfig-3.0"):RegisterOptionsTable("GottaMogEmAll", options, {"gmea"})

-- TreeData
local treeData = {}
local itemTable = {}

-- Labels to display items
local labelTable = {}

-- After loading
function GottaMogEmAll:OnInitialize()
	-- Load database
	self.db = LibStub("AceDB-3.0"):New("GottaMogEmAllDB")
	GottaMogEmAll:Start()
end

function GottaMogEmAll:Start()
	-- Decide whether to show frame or not
	if(not self.db.global.show or self.db.global.show == false) then
		GottaMogEmAll:Hide()
	end

	-- Tree frame
	treeGroup = AceGUI:Create("TreeGroup")
	treeGroup:SetTree(treeData)
	treeGroup:SetFullWidth(true)
	treeGroup:SetFullHeight(true)
	treeGroup:SetLayout("List")
	treeGroup:SetCallback("OnGroupSelected", TreeGroupSelected)
	mainFrame:AddChild(treeGroup)

	button = AceGUI:Create("Button")
	button:SetText("Request items")
	button:SetWidth(200)
	button:SetCallback("OnClick", function()
		GottaMogEmAll:FromJournal()
	end)
	treeGroup:AddChild(button)

	-- Create labels to be filled by item links
	for i = 1, 31, 1 do
		label = AceGUI:Create("InteractiveLabel")
		label:SetColor(1, 1, 1)
		label:SetFont("Fonts\\FRIZQT__.TTF", 12)
		label:SetFullWidth(true)
		label.itemLink = false
		treeGroup:AddChild(label)
		
		label:SetCallback("OnEnter", function(self)
			if(self.itemLink ~= false) then
				SetItemRef(self.itemLink, "ItemLink", "LeftButton")
			end
		end)
		
		label:SetCallback("OnLeave", function(self)
			if(self.itemLink ~= false) then
				SetItemRef(self.itemLink, "ItemLink", "LeftButton")
			end
		end)
		
		table.insert(labelTable, label)
	end
end

-- Draws the loot list (itemlist)
function TreeGroupSelected(container, event, group)
	GottaMogEmAll:ClearLabels()
	
	-- Walk through itemTable and search for items of the clicked boss (group)
	local populatedLabels = 1
	for i = 1, table.getn(itemTable), 1 do
		if(itemTable[i][1] == group) then
			labelTable[populatedLabels]:SetText(itemTable[i][2])
			labelTable[populatedLabels].itemLink = itemTable[i][2]
			populatedLabels = populatedLabels + 1
		end
	end
end

function GottaMogEmAll:ClearLabels()
	-- Reset all labels
	for i = 1, 31, 1 do
		labelTable[i]:SetText("")
		labelTable[i].itemLink = false
	end
end

-- Sets the dungeon to search from the dungeon journal
function GottaMogEmAll:FromJournal()
	-- Check if the EJ is open, else try FromInstance()
	if(not EncounterJournalCloseButton or EncounterJournalCloseButton:IsVisible() == false) then
		GottaMogEmAll:FromInstance()
		return
	end

    -- Load the EJ, Check if a dungeon is selected
    if not EncounterJournal then EncounterJournal_LoadUI() end
    if(EJ_GetInstanceInfo() == 0 or EJ_GetInstanceInfo() == nil) then
        return
    end

   local instanceName, instanceDesc, backgroundTexture, buttonTexture, titleBackground, iconTexture, mapID, instanceLink = EJ_GetInstanceInfo()
   local difficultyID = EJ_GetDifficulty()
   EJ_SetLootFilter(0)
   
   -- Wait for one second, otherwise the item list would not be accessible yet
   C_Timer.After(1, self.CheckItems)
end

-- Runs through the currently filtered items in the EJ
function GottaMogEmAll:CheckItems()
    -- Get number of shown items
    local numberOfItems = EJ_GetNumLoot()
	
	-- Reset interface
	treeData = {}
	treeGroup:SetTree({})
	itemTable = {}
	GottaMogEmAll:ClearLabels()
   
    -- Iterate through items
    local dressableItems = 0
    local newItems = 0
    for  i = 1, numberOfItems, 1 do
        local itemID, encounterID, name, iconID, slot, armorType, itemLink = EJ_GetLootInfoByIndex(i)
        if(IsDressableItem(itemID) == true) then
            local encounterName = EJ_GetEncounterInfo(encounterID)
            local appearanceID, isCollected, sourceID = GottaMogEmAll:GetAppearanceInfo(itemLink)
            if(not isCollected and GottaMogEmAll:PlayerCanCollectAppearance(sourceID) == true and GottaMogEmAll:PlayerHasAppearance(appearanceID) ~= true) then
                newItems = newItems + 1
				table.insert(itemTable, {encounterName, itemLink})
				element = { 
					value = encounterName,
					text = encounterName,
				}
				if(TreeDataContains(treeData, element) == false) then
					table.insert(treeData, element)
				end
            end
            if(GottaMogEmAll:PlayerCanCollectAppearance(sourceID)) then
                dressableItems = dressableItems + 1
			end
        end
    end
	treeGroup:SetTree(treeData)
	mainFrame:SetStatusText(dressableItems .. " equippable, " .. newItems .. " learnable.")
	if(newItems == 0) then
        mainFrame:SetStatusText("Congratulations! You've mogged 'em all!")
		PlaySoundKitID(2859)
		DoEmote("cheer")	
	else
		AceGUI30TreeButton1:Click()
    end
	
	-- If this was called by FromInstance() close the Journal
	if(fromInstance == true) then
		fromInstance = false
		EncounterJournalCloseButton:Click()
	end
	
	GottaMogEmAll:Show()
end

-- Searches the current instance
function GottaMogEmAll:FromInstance()
    local name, instanceType, difficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceMapID, instanceGroupSize = GetInstanceInfo()
	fromInstance = true
	
    -- Open Dungeon Journal and set loot filter to all classes
	if not EncounterJournal then EncounterJournal_LoadUI() end
	if(EJ_GetCurrentInstance() == 0 or EJ_GetCurrentInstance() == nil) then
        return
    end
    EncounterJournal_OpenJournal(difficultyID, EJ_GetCurrentInstance())
	EncounterJournalEncounterFrameInfoLootTab:Click()
    EJ_SetLootFilter(0)

    -- Wait for one second, otherwise the item list would not be accessible yet
    C_Timer.After(1, function() GottaMogEmAll:CheckItems() end)
end

-- Checks if the treeData contins an element
function TreeDataContains(table, element)
  for _, value in pairs(table) do
    if value["value"] == element["value"] then
      return true
    end
  end
  return false
end

-- Returns appearanceID, isCollected and sourceID
function GottaMogEmAll:GetAppearanceInfo(itemLink)
    local itemID, _, _, slotName = GetItemInfoInstant(itemLink)
    if itemLink == itemID then
        itemLink = 'item:' .. itemID
    end
    local slot = InventorySlots[slotName]
    model:SetUnit('player')
    model:Undress()
    model:TryOn(itemLink, slot)
    if(slot == nil) then
	    return false
    end
    local sourceID = model:GetSlotTransmogSources(slot)
    if sourceID then
        local categoryID, appearanceID, canEnchant, texture, isCollected, itemLink = C_TransmogCollection.GetAppearanceSourceInfo(sourceID)
		return appearanceID, isCollected, sourceID
    end
end

function GottaMogEmAll:PlayerCanCollectAppearance(sourceID)
    if(sourceID == nil or sourceID == false) then return false end
	return select(2, C_TransmogCollection.PlayerCanCollectSource(sourceID))
end

-- Looks for items with the same skin
function GottaMogEmAll:PlayerHasAppearance(appearanceID)
    local sources = C_TransmogCollection.GetAppearanceSources(appearanceID)
    if sources then
        for i, source in pairs(sources) do
            if source.isCollected then
                return true
            end
        end
    end
end

-- Config variables
function GottaMogEmAll:ToggleDisplay()
	if(self:IsShown() == true) then
		GottaMogEmAll:Hide()
	else
		GottaMogEmAll:Show()
	end
end

function GottaMogEmAll:Show()
	self.db.global.show = true
	mainFrame:Show()
end

function GottaMogEmAll:Hide()
	self.db.global.show = false
	mainFrame:Hide()
end

function GottaMogEmAll:IsShown()
	return self.db.global.show
end

function GottaMogEmAll:ToggleAutoOpen(info, input)
	if(self:GetAutoOpen() == true) then
		self.db.global.autoOpen = false
	else
		self.db.global.autoOpen = true
	end
	GottaMogEmAll:Print("autoopen set to: " .. tostring(self.db.global.autoOpen))
end

function GottaMogEmAll:GetAutoOpen(info)
	return self.db.global.autoOpen
end